==> verttice_ui-0.0.87.tar.gz: lib/src/widgets/data_factory/form_factory/expand_select_form.dart <==
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/config/config.dart';
import "package:verttice_ui/src/const/box_sizes.dart";
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'forms_factory_index.dart';
import 'package:verttice_ui/src/widgets/snap_builder.dart';
import 'package:verttice_ui/src/widgets/themed_app_bar.dart';
import 'package:verttice_ui/src/widgets/themed_progress.dart';
import 'package:verttice_ui/src/widgets/themed_text_button.dart';
import 'package:verttice_ui/src/widgets/themed_toast.dart';

class ExpandSelectFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final String optionsID;
  final String value;
  final FormFieldState<dynamic>? state;

  const ExpandSelectFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.optionsID,
    this.value = "",
    this.state,
  }) : super(key: key);

  @override
  _ExpandSelectState createState() => _ExpandSelectState();
}

class _ExpandSelectState extends State<ExpandSelectFormFactory> {
  String _value = "";
  String _newOption = "";
  bool _savingNewOption = false;
  bool _isOptionsLoaded = false;
  late StateSetter _setState;

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToString(widget.value);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant ExpandSelectFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = DataClassUtils.dynamicToString(widget.value);
    }
  }

  _addOption(BuildContext context) async {
    try {
      await VertticeUIConfig.instance.setExpanedFormNewOption(
        optionsID: widget.optionsID,
        option: _newOption,
      );
    } catch (e) {
      ThemedToast.error(context: context, message: "Failed to add option");
    } finally {
      _setState(() {
        _isOptionsLoaded = false;
        _savingNewOption = false;
        _newOption = "";
      });
      Navigator.of(context).pop();
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: SelectableText(_value),
      onPressed: () async {
        await showDialog(
          context: context,
          useSafeArea: true,
          builder: (BuildContext context) {
            return SimpleDialog(
              insetPadding: const EdgeInsets.all(6),
              contentPadding: const EdgeInsets.all(6),
              children: [
                SizedBox(
                  height: 300,
                  width: 320,
                  child: Scaffold(
                    appBar: ThemedAppBar(
                      title: "Select ${widget.displayName}",
                      actions: [
                        IconButton(
                          icon: ThemedIcons.add(),
                          onPressed: () async {
                            await showDialog(
                              context: context,
                              useSafeArea: true,
                              builder: (BuildContext context) {
                                _setState = setState;
                                return SimpleDialog(
                                  insetPadding: const EdgeInsets.all(6),
                                  contentPadding: const EdgeInsets.all(6),
                                  children: [
                                    SizedBox(
                                      height: 310,
                                      width: 320,
                                      child: Column(
                                        children: [
                                          ThemedAppBar(
                                            title:
                                                "Add option to ${widget.displayName}",
                                          ),
                                          Expanded(
                                            child: Padding(
                                              padding:
                                                  const EdgeInsets.all(10.0),
                                              child: Column(
                                                children: [
                                                  SizedBox(
                                                    height: 125,
                                                    width: 320,
                                                    child: TextFormFactory(
                                                      value: _newOption,
                                                      displayName: "Option...",
                                                      name: "option",
                                                      onChanged: (name, data) {
                                                        _setState(() {
                                                          _newOption = data;
                                                        });
                                                      },
                                                    ),
                                                  ),
                                                  sizedBoxH16,
                                                  ThemedTextButton(
                                                    "Save option",
                                                    showLoading:
                                                        _savingNewOption,
                                                    onPressed: () async {
                                                      if (_newOption == "") {
                                                        return;
                                                      }

                                                      _setState(() {
                                                        _savingNewOption = true;
                                                      });
                                                      await _addOption(context);
                                                    },
                                                  )
                                                ],
                                              ),
                                            ),
                                          ),
                                        ],
                                      ),
                                    ),
                                  ],
                                );
                              },
                            );
                          },
                        )
                      ],
                    ),
                    body: Center(
                      child: Padding(
                        padding: const EdgeInsets.all(6),
                        child: Builder(
                          builder: (context) {
                            return FutureBuilder<List<String>>(
                              future: VertticeUIConfig.instance
                                  .getExpanedFormOptions(
                                optionsID: widget.optionsID,
                              ),
                              builder: (BuildContext context,
                                  AsyncSnapshot<List<String>> snapshot) {
                                return SnapBuilder<List<String>>(
                                  snapshot: snapshot,
                                  loaded: _isOptionsLoaded,
                                  loadingBuilder: () => const ThemedProgress(),
                                  erroBuilder: (error) => const Text(
                                    'Failed to get options',
                                    style: TextStyle(
                                      color: Colors.red,
                                      fontSize: 20,
                                    ),
                                  ),
                                  successBuilder: (List<String> options) {
                                    _isOptionsLoaded = true;

                                    return SizedBox(
                                      height: 110,
                                      width: 340,
                                      child: SelectFormFactory(
                                        displayName: "Select Option",
                                        name: "selectOption",
                                        onChanged:
                                            (String name, dynamic value) {
                                          widget.onChanged(widget.name, value);
                                          widget.state?.didChange(value);
                                          Navigator.of(context).pop();
                                        },
                                        options: options,
                                      ),
                                    );
                                  },
                                );
                              },
                            );
                          },
                        ),
                      ),
                    ),
                  ),
                )
              ],
            );
          },
        );
      },
    );
  }
}

==> verttice_ui-0.0.97.tar.gz: lib/src/widgets/data_factory/form_factory/expand_select_form.dart <==
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/config/config.dart';
import "package:verttice_ui/src/const/box_sizes.dart";
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'forms_factory_index.dart';
import 'package:verttice_ui/src/widgets/snap_builder.dart';
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

class ExpandSelectFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final String optionsID;
  final String value;
  final FormFieldState<dynamic>? state;

  const ExpandSelectFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.optionsID,
    this.value = "",
    this.state,
  }) : super(key: key);

  @override
  _ExpandSelectState createState() => _ExpandSelectState();
}

class _ExpandSelectState extends State<ExpandSelectFormFactory> {
  String _value = "";
  String _newOption = "";
  bool _savingNewOption = false;
  bool _isOptionsLoaded = false;
  late StateSetter _setState;

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToString(widget.value);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant ExpandSelectFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = DataClassUtils.dynamicToString(widget.value);
    }
  }

  _addOption(BuildContext context) async {
    try {
      await VertticeUIConfig.instance.setExpanedFormNewOption(
        optionsID: widget.optionsID,
        option: _newOption,
      );
    } catch (e) {
      ThemedToast.error(context: context, message: "Failed to add option");
    } finally {
      _setState(() {
        _isOptionsLoaded = false;
        _savingNewOption = false;
        _newOption = "";
      });
      Navigator.of(context).pop();
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: SelectableText(_value),
      onPressed: () async {
        await showDialog(
          context: context,
          useSafeArea: true,
          builder: (BuildContext context) {
            return SimpleDialog(
              insetPadding: const EdgeInsets.all(6),
              contentPadding: const EdgeInsets.all(6),
              children: [
                SizedBox(
                  height: 300,
                  width: 320,
                  child: Scaffold(
                    appBar: ThemedAppBar(
                      title: "Select ${widget.displayName}",
                      actions: [
                        IconButton(
                          icon: ThemedIcons.add(),
                          onPressed: () async {
                            await showDialog(
                              context: context,
                              useSafeArea: true,
                              builder: (BuildContext context) {
                                _setState = setState;
                                return SimpleDialog(
                                  insetPadding: const EdgeInsets.all(6),
                                  contentPadding: const EdgeInsets.all(6),
                                  children: [
                                    SizedBox(
                                      height: 310,
                                      width: 320,
                                      child: Column(
                                        children: [
                                          ThemedAppBar(
                                            title:
                                                "Add option to ${widget.displayName}",
                                          ),
                                          Expanded(
                                            child: Padding(
                                              padding:
                                                  const EdgeInsets.all(10.0),
                                              child: Column(
                                                children: [
                                                  SizedBox(
                                                    height: 125,
                                                    width: 320,
                                                    child: TextFormFactory(
                                                      value: _newOption,
                                                      displayName: "Option...",
                                                      name: "option",
                                                      onChanged: (name, data) {
                                                        _setState(() {
                                                          _newOption = data;
                                                        });
                                                      },
                                                    ),
                                                  ),
                                                  sizedBoxH16,
                                                  ThemedTextButton(
                                                    "Save option",
                                                    showLoading:
                                                        _savingNewOption,
                                                    onPressed: () async {
                                                      if (_newOption == "") {
                                                        return;
                                                      }

                                                      _setState(() {
                                                        _savingNewOption = true;
                                                      });
                                                      await _addOption(context);
                                                    },
                                                  )
                                                ],
                                              ),
                                            ),
                                          ),
                                        ],
                                      ),
                                    ),
                                  ],
                                );
                              },
                            );
                          },
                        )
                      ],
                    ),
                    body: Center(
                      child: Padding(
                        padding: const EdgeInsets.all(6),
                        child: Builder(
                          builder: (context) {
                            return FutureBuilder<List<String>>(
                              future: VertticeUIConfig.instance
                                  .getExpanedFormOptions(
                                optionsID: widget.optionsID,
                              ),
                              builder: (BuildContext context,
                                  AsyncSnapshot<List<String>> snapshot) {
                                return SnapBuilder<List<String>>(
                                  snapshot: snapshot,
                                  loaded: _isOptionsLoaded,
                                  loadingBuilder: () => const ThemedProgress(),
                                  erroBuilder: (error) => const Text(
                                    'Failed to get options',
                                    style: TextStyle(
                                      color: Colors.red,
                                      fontSize: 20,
                                    ),
                                  ),
                                  successBuilder: (List<String> options) {
                                    _isOptionsLoaded = true;

                                    return SizedBox(
                                      height: 110,
                                      width: 340,
                                      child: SelectFormFactory(
                                        displayName: "Select Option",
                                        name: "selectOption",
                                        onChanged:
                                            (String name, dynamic value) {
                                          widget.onChanged(widget.name, value);
                                          widget.state?.didChange(value);
                                          Navigator.of(context).pop();
                                        },
                                        options: options,
                                      ),
                                    );
                                  },
                                );
                              },
                            );
                          },
                        ),
                      ),
                    ),
                  ),
                )
              ],
            );
          },
        );
      },
    );
  }
}
